local ADDON, Package = ...
local Color = Package.Color
local ColorPalette = Package.ColorPalette

local function gametooltip_item(is_item_icon_flagged, is_item_stack_count_flagged, is_item_id_flagged, is_item_level_flagged, is_item_price_flagged)
  local TEXT_LEFTS = {
    GameTooltip = GameTooltipTextLeft1,
    ItemRefTooltip = ItemRefTooltipTextLeft1,
    ShoppingTooltip1 = ShoppingTooltip1TextLeft2,
    ShoppingTooltip2 = ShoppingTooltip2TextLeft2,
  }

  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
    ShoppingTooltip1 = ShoppingTooltip1TextRight2,
    ShoppingTooltip2 = ShoppingTooltip2TextRight2,
  }

  local left_id = nil
  local left_stash = ""

  local right_id = nil
  local right_stash = ""

  local function OnTooltipSetItem(self)
    local _, item_link = self:GetItem()

    if not item_link then
      return nil
    end

    local item_id = GetItemInfoFromHyperlink(item_link)
    local item_name, _, _, item_level, _, _, _, item_stack_count, _, item_icon, item_sell_price = GetItemInfo(item_id)

    if not item_name then
      return nil
    end

    local left = TEXT_LEFTS[self:GetName()]
    local left_text = left:GetText() or ""

    local right = TEXT_RIGHTS[self:GetName()]
    local right_text = right:GetText() or ""

    if left_id ~= item_id then
      if is_item_icon_flagged then
        local item_icon_markup = CreateTextureMarkup(item_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

        left_text = item_icon_markup .. " " ..left_text
      end

      if is_item_stack_count_flagged and item_stack_count > 1 then
        left_text = left_text .. " (" .. item_stack_count .. ")"
      end

      left_id = item_id
      left_stash = left_text
    end

    left:SetText(left_stash)

    if right_id ~= item_id then
      if is_item_id_flagged then
        right_text = right_text .. Color.format(ColorPalette.GRAY, item_id)
      end

      if is_item_level_flagged then
        right_text = right_text .. Color.format(ColorPalette.GOLD, " " .. LEVEL .. " " .. item_level)
      end

      right_id = item_id
      right_stash = right_text
    end

    right:SetText(right_stash)
    right:Show()

    if is_item_price_flagged then
      local container = GetMouseFocus()
      local item_count = 1

      if type(container) == "table" and type(container.count) == "number" then
        item_count = container.count
      end

      if item_count and item_count > 0 and item_sell_price and item_sell_price > 0 and self.shownMoneyFrames == nil then
        SetTooltipMoney(self, item_count * item_sell_price, "STATIC", SELL_PRICE .. ":")
      end
    end
  end

  if is_item_icon_flagged or is_item_stack_count_flagged or is_item_id_flagged or is_item_level_flagged or is_item_price_flagged then
    GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ItemRefTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ShoppingTooltip1:HookScript("OnTooltipSetItem", OnTooltipSetItem)
    ShoppingTooltip2:HookScript("OnTooltipSetItem", OnTooltipSetItem)
  end
end

local function gametooltip_spell(is_spell_icon_flagged, is_spell_id_flagged)
  local TEXT_LEFTS = {
    GameTooltip = GameTooltipTextLeft1,
    ItemRefTooltip = ItemRefTooltipTextLeft1,
  }

  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
  }

  local left_id = nil
  local left_stash = ""

  local right_id = nil
  local right_stash = ""

  local function OnTooltipSetSpell(self)
    local _, spell_id = self:GetSpell()

    if not spell_id then
      return nil
    end

    local left = TEXT_LEFTS[self:GetName()]
    local left_text = left:GetText() or ""

    local right = TEXT_RIGHTS[self:GetName()]
    local right_text = right:GetText() or ""

    if is_spell_icon_flagged and not left_id ~= spell_id then
      local spell_icon = GetSpellTexture(spell_id)
      local spell_icon_markup = CreateTextureMarkup(spell_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

      left_id = spell_id
      left_stash = spell_icon_markup .. " " .. left_text
    end

    left:SetText(left_stash)

    if is_spell_id_flagged and not right_id ~= spell_id then
      local spell_id_text = Color.format(ColorPalette.GRAY, spell_id)

      right_id = spell_id
      right_stash = spell_id_text .. " " .. right_text
    end

    right:SetText(right_stash)
    right:Show()
  end

  if is_spell_icon_flagged or is_spell_id_flagged then
    GameTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetSpell)
    ItemRefTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetSpell)
  end
end

local function gametooltip_aura(is_aura_id_flagged, is_aura_icon_flagged, is_aura_caster_flagged)
  local left = GameTooltipTextLeft1
  local right = GameTooltipTextRight1

  local function UpdateTooltip(self)
    if GameTooltip:IsOwned(self) then
      GameTooltip:SetUnitAura(self.unit, self:GetID(), self.filter)
    end
  end

  local function SetUnitAura(self, unit_id, slot_id, type)
    local _, aura_icon, _, _, _, _, aura_caster_id, _, _, aura_id = UnitAura(unit_id, slot_id, type)

    local left_text = left:GetText() or ""
    local right_text = right:GetText() or ""

    if is_aura_icon_flagged and aura_icon then
      local aura_icon_markup = CreateTextureMarkup(aura_icon or 134400, 64, 64, 16, 16, 0.08, 0.92, 0.08, 0.92, 0, 1)

      left_text = aura_icon_markup .. " " .. left_text
    end

    if is_aura_caster_flagged and aura_caster_id then
      local aura_caster_name = UnitName(aura_caster_id)
      local aura_caster_class = UnitClassBase(aura_caster_id)

      if aura_caster_name and aura_caster_class then
        left_text = left_text .. RAID_CLASS_COLORS[aura_caster_class]:WrapTextInColorCode(" <" .. aura_caster_name .. ">")
      end
    end

    left:SetText(left_text)

    if is_aura_id_flagged and aura_id then
      right_text = Color.format(ColorPalette.GRAY, aura_id) .. " " .. right_text
    end

    right:SetText(right_text)
    right:Show()

    GameTooltip:Show()

    local button_name = (type == "HELPFUL" and "BuffButton") or (type == "HARMFUL" and "DebuffButton") or nil
    local button = nil

    if button_name then
      button = _G[button_name .. slot_id]
    end

    if button and not button:GetScript("OnUpdate") then
      if not button.UpdateTooltip then
        button.UpdateTooltip = UpdateTooltip
      end
    else
      button.UpdateTooltip = nil
    end
  end

  if is_aura_id_flagged or is_aura_icon_flagged or is_aura_caster_flagged then
    hooksecurefunc(GameTooltip, "SetUnitAura", SetUnitAura)
  end
end

Package.gametooltip_item = gametooltip_item
Package.gametooltip_spell = gametooltip_spell
Package.gametooltip_aura = gametooltip_aura
